#ifndef CLUSTER__H
#define CLUSTER__H

#include <vector>
#include <iostream>
using namespace std;

template <class T>
struct Cluster{
    T prototype;
    vector<T*> members;

    void step(){
        if(members.empty())
            return;
        
        T mean = *members[0];
        for(int i =1; i < members.size(); ++i){
            mean += *(members[i]);
        }
        mean /= members.size();
        prototype = mean;
    }
    
    double error() {
        double rv = 0;
        for(T* t: members){
            rv += (prototype - *t);
        }
        return rv;
    }
};

template<class T>
ostream& operator<<(ostream& os, Cluster<T>& cluster){
    os << cluster.members.size() << endl;
    os << cluster.error() << endl;
    os << cluster.prototype << endl;
    return os;
}


template<class T>
struct ClusterAlgorithm{
    vector<Cluster<T>> clusters;
    vector<T*> data;
    
    ClusterAlgorithm(int k, vector<T*> d){
        clusters = vector<Cluster<T>>(k);
        data = d;
        for(int i =0; i < clusters.size(); ++i){
            clusters[i].prototype = *(data[i]);

        }
    }
    
    void lloyds(int n){
        double error = 0;
        for(int k =0; k < n; ++k){
            for(Cluster<T>& c: clusters){
                c.members.clear();
            }
            
            for(T* t: data){
                double small = clusters[0].prototype-*t;
                int index = 0;
                for(int i =1; i < clusters.size(); ++i){
                    double val = clusters[i].prototype-*t;
                    if(val < small){
                        small = val;
                        index = i;
                    }
                }
                clusters[index].members.push_back(t);
            }
            
            for(Cluster<T>& c: clusters){
                c.step();
                error += c.error();
            }
        }
    }
    
    
};


#endif

