#include <iostream>
#include <cmath>
#include <vector>
#include <sstream>
#include <fstream>
#include "Cluster.h"
using namespace std;

struct Coord{
	double lat;
	double lon;

	Coord& operator/=(double rhs){
		lat /= rhs;
		lon /= rhs;
		return *this;
	} 

    Coord& operator+=(const Coord& rhs){
        lat += rhs.lat;
        lon += rhs.lon;
        return *this;
    }

    //return distance between two Coords
    double operator-(const Coord& rhs){
        double x = lat-rhs.lat;
        double y = lon-rhs.lon;
        return sqrt(x*x + y*y);
    }
    
};

ostream& operator<<(ostream& os, Coord& c){
    os << "(" << c.lat << "," << c.lon << ")";
    return os;
}

struct Storm{
	int year;
	string name;
    string id;
	bool hurricane = false;
    bool landing = false;
    vector<Coord> pos;

    double operator-(const Storm& rhs){
        double error = 0;
        for(int i =0; i < pos.size(); ++i){
            error += (pos[i] - rhs.pos[i]);
        }
        return error;
    }
    
    Storm& operator/=(double d){
        for(Coord& c: pos){
            c /= d;
        }
        return *this;
    }

    Storm& operator+=(const Storm& rhs){
        for(int i =0; i < pos.size(); ++i){
            pos[i] += rhs.pos[i];
        }
        return *this;
    }
    
    void scale(int spots){
        vector<Coord> result;
        result.push_back(pos[0]);
        for(int i = 1; i < spots-1; ++i){
            double alpha = ((double)i)/(double)(spots-1);
            result.push_back(pos[alpha*(pos.size()-1)]);
        }
        result.push_back(pos[pos.size()-1]);
        pos = result;
    }
    

    
};

ostream& operator<<(ostream& os, Storm& storm){
    os << storm.pos.size() << endl;
    for(Coord c :storm.pos){
        os << c << " ";
    }
    os << endl;
    return os;
}


vector<string> split(string input){
	stringstream sin;
	sin.str(input);
	vector<string> rv;
    string sub;
    string clean;
	while(sin.good()){
		getline(sin,sub,',');
        stringstream white;
        white.str(sub);
        white >> clean;
		rv.push_back(clean);
	}
	return rv;
}



vector<Storm*> readFile(string filename){
    vector<Storm*> rv;
    ifstream fin;
    fin.open(filename);
    string header;
    while(getline(fin,header)){
        vector<string> tokens = split(header);
		Storm* storm = new Storm;
        storm->id = tokens[0];
        storm->name = tokens[1];
        int lines = stoi(tokens[2]);
        for(int i =0; i < lines; ++i){
            getline(fin,header);
            tokens = split(header);
			Coord c = {0,0};
            storm->landing = storm->landing || (tokens[2] == "L");
            storm->hurricane = storm->hurricane || (tokens[3] == "HU");
			c.lat = stof(tokens[4].substr(0,tokens[4].size()-1));
			c.lon = stof(tokens[5].substr(0,tokens[5].size()-1)); 
			if(tokens[4][tokens[4].size()-1] == 'S'){
				c.lat *= -1;
			}
			if(tokens[5][tokens[5].size()-1] == 'W'){
                c.lon *= -1;
			}
			storm->pos.push_back(c);
        }
        storm->scale(7);
        if(storm->hurricane && storm->landing){
            rv.push_back(storm);
        }else{
            delete storm;
        }
    }
    fin.close();
    return rv;
}

double distanceAtPoint(Cluster<Storm>& cluster,int ndx){
    double rv = 0;
    for(Storm* storm: cluster.members){
        rv = max(rv, cluster.prototype.pos[ndx]-storm->pos[ndx]);
    }
    return rv;
}

int main(){
	//read in tracks
	vector<Storm*> storms = readFile("hurdat2.txt");

    ClusterAlgorithm<Storm> alg(5,storms);
    alg.lloyds(1000);

    int points = storms[0]->pos.size();
    
    for(int i =0; i < points; ++i){
        for(Cluster<Storm> c: alg.clusters){
           cout << c.prototype.pos[i].lon << "\t" << c.prototype.pos[i].lat << "\t" << distanceAtPoint(c,i) << "\t";
        }
        cout << endl;
    }

    
    //for(Cluster<Storm> c: alg.clusters){
    //    cout << c << endl;
    //}
    
	return 0;
}
